@@ -49,6 +49,7 @@ define([
             this.id = this.state.config.id;
             this.name = this.state.config.name;
             this.path = this.state.config.path;
+            
 
             this.config = {
                 sizeLevel: 0,          // 0: 400x400 / 1: 500x500 / 2: 600x500 / 3: 750x500
@@ -65,7 +66,8 @@ define([
                 footer: true,
                 position: { right: 10, top: 120 },
                 size: { width: 400, height: 550 },
-                saveOnly: false
+                saveOnly: false,
+                checkModules: [] // module aliases or function names
             };
 
             // check BoardFrame width and set initial position of popup
@@ -301,7 +303,7 @@ define([
                 // add install codes
                 var codes = that.generateInstallCode();
                 codes && codes.forEach(code => {
-                    com_interface.insertCell('code', code);
+                    com_interface.insertCell('code', code, true, that.getSigText());
                 });
             });
 
@@ -375,8 +377,10 @@ define([
                         }
                         break;
                     case 'run':
-                        that.save();
-                        that.run();
+                        let result = that.run();
+                        if (result) {
+                            that.save();
+                        }
                         break;
                     case 'show-detail':
                         $(that.wrapSelector('.vp-popup-run-detailbox')).show();
@@ -395,14 +399,15 @@ define([
                         that.save();
                         break;
                     case 'add':
-                        that.save();
-                        that.run(false);
+                        let result = that.run(false);
+                        if (result) {
+                            that.save();
+                        }
                         break;
                 }
             });
             // Close event for inner popup
             $(this.wrapSelector('.vp-inner-popup-close')).on('click', function(evt) {
-                that.handleInnerCancel();
                 that.closeInnerPopup();
             });
             // Click button event for inner popup
@@ -410,7 +415,6 @@ define([
                 let btnType = $(this).data('type');
                 switch(btnType) {
                     case 'cancel':
-                        that.handleInnerCancel();
                         that.closeInnerPopup();
                         break;
                     case 'ok':
@@ -479,7 +483,7 @@ define([
         template() { 
             this.$pageDom = $(popupComponentHtml);
             // set title
-            this.$pageDom.find('.vp-popup-title').text(this.state.config.name);
+            this.$pageDom.find('.vp-popup-title').text(this.name);
             // set body
             this.$pageDom.find('.vp-popup-content').html(this.templateForBody());
             return this.$pageDom;
@@ -680,24 +684,101 @@ define([
             }
         }
 
-        run(execute=true, addcell=true) {
-            let code = this.generateCode();
-            let mode = this.config.executeMode;
-            let blockNumber = -1;
-            // check if it's block
+        getSigText() {
+            let sigText = '';
             if (this.getTaskType() == 'block') {
                 let block = this.taskItem;
-                blockNumber = block.blockNumber;
+                sigText = block.sigText;
+            } else {
+                try {
+                    let menuGroup = this.path.split(' - ')[1];
+                    let menuGroupLabel = vpConfig.getMenuGroupLabel(menuGroup);
+                    if (menuGroupLabel != undefined && menuGroupLabel !== '') {
+                        sigText = menuGroupLabel + ' > ' + this.name;
+                    } else {
+                        sigText = this.name;
+                    }
+                } catch {}
             }
-            if (addcell) {
-                if (Array.isArray(code)) {
-                    // insert cells if it's array of codes
-                    com_interface.insertCells(mode, code, execute, blockNumber);
-                } else {
-                    com_interface.insertCell(mode, code, execute, blockNumber);
+            return sigText;
+        }
+
+        /**
+         * Check if required option is filled
+         * @returns true if it's ok / false if there is empty required option
+         */
+        checkRequiredOption() {
+            let requiredFilled = true;
+            let requiredTags = $(this.wrapSelector('input[required=true],input[required=required]'));
+
+            if (requiredTags) {
+                for (let i = 0; i < requiredTags.length; i++) {
+                    let thisTag = $(requiredTags[i]);
+                    // if it's visible and empty, focus on it
+                    if (thisTag.is(':visible') && thisTag.val() == '') {
+                        $(requiredTags[i]).focus();
+                        requiredFilled = false;
+                        break;
+                    }
                 }
             }
-            return code;
+
+            return requiredFilled;
+        }
+
+        checkAndRunModules(execute=true, background=false) {
+            let sigText = this.getSigText();
+
+            let checkModules = this.config.checkModules;
+            return new Promise(function(resolve, reject) {
+                if (checkModules.length > 0) {
+                    vpKernel.checkModule(checkModules).then(function(resultObj) {
+                        let { result } = resultObj;
+                        let checkedList = JSON.parse(result);
+                        let executeList = [];
+                        checkedList && checkedList.forEach((mod, idx) => {
+                            if (mod == false) {
+                                let modInfo = vpConfig.getModuleCode(checkModules[idx]);
+                                if (modInfo) {
+                                    executeList.push(modInfo.code);
+                                }
+                            }
+                        });
+                        if (executeList && executeList.length > 0) {
+                            com_interface.insertCell('code', executeList.join('\n'), execute, sigText);
+                        }
+                        resolve(executeList);
+                    });
+                } else {
+                    resolve([]);
+                }
+            });
+        }
+
+        run(execute=true, addcell=true) {
+            // check required
+            if (this.checkRequiredOption() === false) {
+                return false;
+            }
+
+            let mode = this.config.executeMode;
+            let sigText = this.getSigText();
+            let code = this.generateCode();
+
+            vpLog.display(VP_LOG_TYPE.DEVELOP, sigText, mode, code);
+
+            // check modules
+            this.checkAndRunModules(execute).then(function(executeList) {
+                if (addcell) {
+                    if (Array.isArray(code)) {
+                        // insert cells if it's array of codes
+                        com_interface.insertCells(mode, code, execute, sigText);
+                    } else {
+                        com_interface.insertCell(mode, code, execute, sigText);
+                    }
+                }
+            });
+            return true;
         }
 
         /**
@@ -837,13 +918,16 @@ define([
             $(this.wrapSelector('.vp-inner-popup-box')).show();
 
             // focus on first input
-            $(this.wrapSelector('.vp-inner-popup-box input[type=text]:not(:disabled):visible:first')).focus();
+            $(this.wrapSelector('.vp-inner-popup-box input:not(:disabled):visible:first')).focus();
+            // disable Jupyter key
+            com_interface.disableOtherShortcut();
         }
         
         /**
          * Close inner popup box
          */
         closeInnerPopup() {
+            this.handleInnerCancel();
             $(this.wrapSelector('.vp-inner-popup-box')).hide();
         }
 
