@@ -30,6 +30,7 @@ define([
         _init() {
             super._init();
             this.config.sizeLevel = 3;
+            this.config.checkModules = ['pd'];
 
             // state
             this.state = {
@@ -107,12 +108,13 @@ define([
                     that.state.originObj = origin;
                     that.state.tempObj = '_vp';
                     that.state.returnObj = that.state.tempObj;
-                    that.state.inplace = false;
+                    if (that.state.inplace === true) {
+                        that.state.returnObj = origin;
+                    }
                     that.initState();
 
                     // reset return obj
-                    $(that.wrapSelector('#vp_feReturn')).val(that.state.tempObj);
-                    $(that.wrapSelector('#inplace')).prop('checked', false);
+                    $(that.wrapSelector('#vp_feReturn')).val(that.state.returnObj);
     
                     // reset table
                     $(that.wrapSelector('.' + VP_FE_TABLE)).replaceWith(function() {
@@ -368,6 +370,12 @@ define([
                     case FRAME_EDIT_TYPE.AS_TYPE:
                         that.openInputPopup(editType);
                         break;
+                    case FRAME_EDIT_TYPE.DROP_OUT:
+                        that.config.checkModules = ['pd', 'np', 'vp_drop_outlier'];
+                        that.checkAndRunModules(true).then(function() {
+                            that.loadCode(that.getTypeCode(editType));
+                        });
+                        break;
                     default:
                         that.loadCode(that.getTypeCode(editType));
                         break;
@@ -514,6 +522,7 @@ define([
             super.render();
 
             var {
+                originObj,
                 returnObj,
                 inplace,
                 steps
@@ -521,7 +530,7 @@ define([
 
             this.loadVariableList();
     
-            $(this.wrapSelector('#vp_feVariable')).val(this.state.originObj);
+            $(this.wrapSelector('#vp_feVariable')).val(originObj);
     
             $(this.wrapSelector('#vp_feReturn')).val(returnObj);
 
@@ -543,6 +552,7 @@ define([
             variableInput.addClass('vp-state');
             variableInput.setPlaceholder('Select variable');
             variableInput.setSuggestList(function () { return mappedList; });
+            variableInput.addAttribute('required', true);
             variableInput.setSelectEvent(function (value) {
                 $(this.wrapSelector()).val(value);
                 $(this.wrapSelector()).trigger('change');
@@ -663,7 +673,12 @@ define([
             });
         }
 
-        // Render Inner popup page
+        /**
+         * Render Inner popup page
+         * @param {*} type 
+         * @param {*} targetLabel 
+         * @returns 
+         */
         renderAddPage = function(type, targetLabel = '') {
             var content = new com_String();
             content.appendFormatLine('<div class="{0}">', 'vp-inner-popup-addpage');
@@ -682,7 +697,9 @@ define([
             content.appendFormatLine('<td><select class="{0}">', 'vp-inner-popup-addtype');
             content.appendFormatLine('<option value="{0}">{1}</option>', 'value', 'Value');
             content.appendFormatLine('<option value="{0}">{1}</option>', 'calculation', 'Calculation');
-            content.appendFormatLine('<option value="{0}">{1}</option>', 'replace', 'Replace');
+            if (type == 'replace') {
+                content.appendFormatLine('<option value="{0}">{1}</option>', 'replace', 'Replace');
+            }
             if (type == 'column' || type == 'replace') {
                 content.appendFormatLine('<option value="{0}">{1}</option>', 'subset', 'Subset');
             }
@@ -1211,6 +1228,7 @@ define([
             switch (type) {
                 case FRAME_EDIT_TYPE.INIT:
                     code.appendFormat('{0} = {1}.copy()', tempObj, orgObj);
+                    this.config.checkModules = ['pd'];
                     break;
                 case FRAME_EDIT_TYPE.DROP:
                     code.appendFormat("{0}.drop([{1}], axis={2}, inplace=True)", tempObj, selectedName, axis);
@@ -1245,6 +1263,17 @@ define([
                         code.appendFormat("{0} = vp_drop_outlier({1}, {2})", tempObj, tempObj, selectedName);
                     }
                     break;
+                case FRAME_EDIT_TYPE.LABEL_ENCODING:
+                    if (axis == FRAME_AXIS.COLUMN) {
+                        let encodedColName = this.state.selected.map(col=> { 
+                            if (col.code !== col.label) {
+                                return com_util.formatString("'{0}'", col.label + '_label');
+                            }
+                            return col.label + '_label' 
+                        }).join(',');
+                        code.appendFormat("{0}[{1}] = pd.Categorical({2}[{3}]).codes", tempObj, encodedColName, tempObj, selectedName);
+                    }
+                    break;
                 case FRAME_EDIT_TYPE.ONE_HOT_ENCODING:
                     if (axis == FRAME_AXIS.COLUMN) {
                         code.appendFormat("{0} = pd.get_dummies(data={1}, columns=[{2}])", tempObj, tempObj, selectedName);
@@ -1347,7 +1376,7 @@ define([
     
             var code = new com_String();
             code.appendLine(codeStr);
-            code.appendFormat("{0}[{1}:{2}].to_json(orient='{3}')", tempObj, prevLines, lines, 'split');
+            code.appendFormat("{0}.iloc[{1}:{2}].to_json(orient='{3}')", tempObj, prevLines, lines, 'split');
             
             this.loading = true;
             vpKernel.execute(code.toString()).then(function(resultObj) {
@@ -1467,7 +1496,11 @@ define([
                             that.loadInfo();
                             // add to stack
                             if (codeStr !== '') {
-                                that.state.steps.push(codeStr);
+                                let newSteps = codeStr.split('\n');
+                                that.state.steps = [
+                                    ...that.state.steps,
+                                    ...newSteps
+                                ]
                                 var replacedCode = codeStr.replaceAll(that.state.tempObj, that.state.returnObj);
                                 that.setPreview(replacedCode);
                             }
@@ -1490,8 +1523,11 @@ define([
                 }
             }).catch(function(resultObj) {
                 let { result, type, msg } = resultObj;
-                vpLog.display(VP_LOG_TYPE.ERROR, result.ename + ': ' + result.evalue, msg);
-                com_util.renderAlertModal(result.ename + ': ' + result.evalue);
+                vpLog.display(VP_LOG_TYPE.ERROR, result.ename + ': ' + result.evalue, msg, code.toString());
+                if (that.isHidden() == false) {
+                    // show alert modal only if this popup is visible
+                    com_util.renderAlertModal(result.ename + ': ' + result.evalue);
+                }
                 that.loading = false;
             });
     
@@ -1579,6 +1615,8 @@ define([
         DROP_OUT: 11, 
 
         ONE_HOT_ENCODING: 6,
+        LABEL_ENCODING: 12,
+
         SET_IDX: 7,
         RESET_IDX: 8,
         REPLACE: 9,
