@@ -31,6 +31,7 @@ define([
 
             this.config.dataview = false;
             this.config.size = { width: 1064, height: 550 };
+            this.config.checkModules = ['plt', 'sns'];
 
             this.state = {
                 chartType: 'scatterplot',
@@ -49,6 +50,11 @@ define([
                 kde: '',
                 stat: '',
                 showValues: false,
+                showValuesPrecision: '',
+                sortBy: 'y',
+                sortType: '',
+                sortHue: '',
+                sortHueText: false,
                 // axes options
                 x_limit_from: '',
                 x_limit_to: '',
@@ -193,8 +199,10 @@ define([
                     $(that.wrapSelector('#stat')).closest('.sb-option').show();
                 } else if (chartType == 'barplot') {
                     $(that.wrapSelector('#showValues')).closest('.sb-option').show();
+                    $(that.wrapSelector('#sortBy')).closest('.sb-option').show();
                 } else if (chartType == 'countplot') {
                     $(that.wrapSelector('#showValues')).closest('.sb-option').show();
+                    $(that.wrapSelector('#sortBy')).closest('.sb-option').show();
                 }
             });
             
@@ -208,6 +216,11 @@ define([
                     $(that.wrapSelector('#x')).closest('.vp-ds-box').replaceWith('<select id="x"></select>');
                     $(that.wrapSelector('#y')).closest('.vp-ds-box').replaceWith('<select id="y"></select>');
                     $(that.wrapSelector('#hue')).closest('.vp-ds-box').replaceWith('<select id="hue"></select>');
+
+                    // FIXME: hide sort values for barplot/countplot (as temporary)
+                    if (that.state.chartType == 'barplot' || that.state.chartType == 'countplot') {
+                        $(that.wrapSelector('#sortBy')).closest('.sb-option').show();
+                    }
                 } else {
                     // set X Y indivisually
                     // disable data selection
@@ -226,10 +239,55 @@ define([
 
                     let dataSelectorHue = new DataSelector({ pageThis: that, id: 'hue' });
                     $(that.wrapSelector('#hue')).replaceWith(dataSelectorHue.toTagString());
+
+                    // FIXME: hide sort values for barplot/countplot (as temporary)
+                    if (that.state.chartType == 'barplot' || that.state.chartType == 'countplot') {
+                        $(that.wrapSelector('#sortBy')).closest('.sb-option').hide();
+                    }
                     
                 }
             });
 
+            // change hue
+            $(document).off('change', this.wrapSelector('.vp-state'));
+            $(document).on('change', this.wrapSelector('#hue'), function() {
+                let { chartType, data } = that.state;
+                let hue = $(this).val();
+                if (chartType == 'barplot' || chartType == 'countplot') {
+                    let colDtype = $(that.wrapSelector('#hue')).find('option:selected').data('type');
+                    console.log(data, hue);
+                    // get result and load column list
+                    vpKernel.getColumnCategory(data, hue).then(function (resultObj) {
+                        let { result } = resultObj;
+                        try {
+                            var category = JSON.parse(result);
+                            if (category && category.length > 0 && colDtype == 'object') {
+                                // if it's categorical column and its dtype is object, check 'Text' as default
+                                $(that.wrapSelector('#sortHueText')).prop('checked', true);
+                            } else {
+                                $(that.wrapSelector('#sortHueText')).prop('checked', false);
+                            }
+                            $(that.wrapSelector('#sortHue')).replaceWith(that.templateForHueCondition(category, colDtype));
+                        } catch {
+                            $(that.wrapSelector('#sortHueText')).prop('checked', false);
+                            $(that.wrapSelector('#sortHue')).replaceWith(that.templateForHueCondition([], colDtype));
+                        }
+                    });
+                }
+            });
+
+            // show values or not
+            $(this.wrapSelector('#showValues')).on('change', function() {
+                let checked = $(this).prop('checked');
+                if (checked === true) {
+                    that.config.checkModules = ['plt', 'sns', 'np', 'vp_seaborn_show_values'];
+                    $(that.wrapSelector('#showValuesPrecision')).attr('disabled', false);
+                } else {
+                    that.config.checkModules = ['plt', 'sns'];
+                    $(that.wrapSelector('#showValuesPrecision')).attr('disabled', true);
+                }
+            });
+
             // use color or not
             $(this.wrapSelector('#useColor')).on('change', function() {
                 that.state.useColor = $(this).prop('checked');
@@ -526,7 +584,7 @@ define([
                 key: userCodeKey,
                 selector: userCodeTarget,
                 events: [{
-                    key: 'change',
+                    key: 'blur',
                     callback: function(instance, evt) {
                         // save its state
                         instance.save();
@@ -540,6 +598,25 @@ define([
             this.loadPreview();
         }
 
+        templateForHueCondition(category, dtype='object') {
+            var vpCondSuggest = new SuggestInput();
+            vpCondSuggest.setComponentID('sortHue');
+            vpCondSuggest.addClass('vp-input vp-state');
+            vpCondSuggest.setPlaceholder('Type hue condition');
+            if (category && category.length > 0) {
+                vpCondSuggest.setPlaceholder((dtype=='object'?'Categorical':dtype) + " dtype");
+                vpCondSuggest.setSuggestList(function () { return category; });
+                vpCondSuggest.setSelectEvent(function (value) {
+                    $(this.wrapSelector()).val(value);
+                    $(this.wrapSelector()).trigger('change');
+                });
+                vpCondSuggest.setNormalFilter(false);
+            } else {
+                vpCondSuggest.setPlaceholder(dtype==''?'Value':(dtype + " dtype"));
+            }
+            return vpCondSuggest.toTagString();
+        }
+
         bindSettingBox() {
             //====================================================================
             // Stylesheet suggestinput
@@ -625,44 +702,46 @@ define([
             let that = this;
             let code = this.generateCode(true);
 
-            // show variable information on clicking variable
-            vpKernel.execute(code).then(function(resultObj) {
-                let { result, type, msg } = resultObj;
-                if (msg.content.data) {
-                    var textResult = msg.content.data["text/plain"];
-                    var htmlResult = msg.content.data["text/html"];
-                    var imgResult = msg.content.data["image/png"];
-                    
-                    $(that.wrapSelector('#chartPreview')).html('');
-                    if (htmlResult != undefined) {
-                        // 1. HTML tag
-                        $(that.wrapSelector('#chartPreview')).append(htmlResult);
-                    } else if (imgResult != undefined) {
-                        // 2. Image data (base64)
-                        var imgTag = '<img src="data:image/png;base64, ' + imgResult + '">';
-                        $(that.wrapSelector('#chartPreview')).append(imgTag);
-                    } else if (textResult != undefined) {
-                        // 3. Text data
-                        var preTag = document.createElement('pre');
-                        $(preTag).text(textResult);
-                        $(that.wrapSelector('#chartPreview')).html(preTag);
+            that.checkAndRunModules(true).then(function() {
+                // show variable information on clicking variable
+                vpKernel.execute(code).then(function(resultObj) {
+                    let { result, type, msg } = resultObj;
+                    if (msg.content.data) {
+                        var textResult = msg.content.data["text/plain"];
+                        var htmlResult = msg.content.data["text/html"];
+                        var imgResult = msg.content.data["image/png"];
+                        
+                        $(that.wrapSelector('#chartPreview')).html('');
+                        if (htmlResult != undefined) {
+                            // 1. HTML tag
+                            $(that.wrapSelector('#chartPreview')).append(htmlResult);
+                        } else if (imgResult != undefined) {
+                            // 2. Image data (base64)
+                            var imgTag = '<img src="data:image/png;base64, ' + imgResult + '">';
+                            $(that.wrapSelector('#chartPreview')).append(imgTag);
+                        } else if (textResult != undefined) {
+                            // 3. Text data
+                            var preTag = document.createElement('pre');
+                            $(preTag).text(textResult);
+                            $(that.wrapSelector('#chartPreview')).html(preTag);
+                        }
+                    } else {
+                        var errorContent = '';
+                        if (msg.content.ename) {
+                            errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue);
+                        }
+                        $(that.wrapSelector('#chartPreview')).html(errorContent);
+                        vpLog.display(VP_LOG_TYPE.ERROR, msg.content.ename, msg.content.evalue, msg.content);
                     }
-                } else {
+                }).catch(function(resultObj) {
+                    let { msg } = resultObj;
                     var errorContent = '';
                     if (msg.content.ename) {
                         errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue);
                     }
                     $(that.wrapSelector('#chartPreview')).html(errorContent);
                     vpLog.display(VP_LOG_TYPE.ERROR, msg.content.ename, msg.content.evalue, msg.content);
-                }
-            }).catch(function(resultObj) {
-                let { msg } = resultObj;
-                var errorContent = '';
-                if (msg.content.ename) {
-                    errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue);
-                }
-                $(that.wrapSelector('#chartPreview')).html(errorContent);
-                vpLog.display(VP_LOG_TYPE.ERROR, msg.content.ename, msg.content.evalue, msg.content);
+                });
             });
         }
 
@@ -705,7 +784,10 @@ define([
 
         generateCode(preview=false) {
             let { 
-                chartType, data, x, y, setXY, hue, kde, stat, showValues, userOption='', 
+                chartType, data, x, y, setXY, hue, kde, stat, 
+                showValues, showValuesPrecision, 
+                sortType, sortBy, sortHue, sortHueText,
+                userOption='', 
                 x_limit_from, x_limit_to, y_limit_from, y_limit_to,
                 xticks, xticks_label, xticks_rotate, removeXticks,
                 yticks, yticks_label, yticks_rotate, removeYticks,
@@ -720,25 +802,6 @@ define([
             let config = this.chartConfig[chartType];
             let state = JSON.parse(JSON.stringify(this.state));
 
-            let chartCode = new com_String();
-
-            let etcOptionCode = []
-            if (useColor == true && color != '') {
-                etcOptionCode.push(com_util.formatString("color='{0}'", color));
-            }
-            if (markerStyle != '') {
-                // TODO: marker to seaborn argument (ex. marker='+' / markers={'Lunch':'s', 'Dinner':'X'})
-                etcOptionCode.push(com_util.formatString("marker='{0}'", markerStyle));
-            }
-            if (showValues === true && chartType === 'barplot') {
-                etcOptionCode.push('ci=None');
-            }
-
-            // add user option
-            if (userOption != '') {
-                etcOptionCode.push(userOption);
-            }
-
             if (preview && useSampling) {
                 // data sampling code for preview
                 // convertedData = data + '.sample(n=' + sampleCount + ', random_state=0)';
@@ -760,7 +823,71 @@ define([
                         state.data = com_util.formatString('_vp_sample({0}, {1})', data, sampleCount);
                     }
                 }
-            }   
+            }  
+
+            let chartCode = new com_String();
+
+            let etcOptionCode = []
+            if (useColor == true && color != '') {
+                etcOptionCode.push(com_util.formatString("color='{0}'", color));
+            }
+            if (markerStyle != '') {
+                // TODO: marker to seaborn argument (ex. marker='+' / markers={'Lunch':'s', 'Dinner':'X'})
+                etcOptionCode.push(com_util.formatString("marker='{0}'", markerStyle));
+            }
+            if (showValues === true && chartType === 'barplot') {
+                etcOptionCode.push('ci=None');
+            }
+            if (sortType != '') {
+                let sortCode = '';
+                let sortTypeStr = (sortType === 'descending'? 'ascending=False': 'ascending=True');
+                let sortX = state.x;
+                let sortY = state.y;
+                if (sortBy === 'x') {
+                    sortX = state.y;
+                    sortY = state.x;
+                }
+                if (chartType === 'barplot') {
+                    if (setXY === true) {
+                        // TODO: sort on setXY
+                        // if (hue !== '' && sortHue !== '') {
+                        //     sortCode = com_util.formatString("{}.groupby({})[{}].mean().sort_values({}).index")
+                        // } else {
+                        //     sortCode = com_util.formatString("pd.concat([{0},{1}], axis=1).groupby({2})[{3}].mean().sort_values({4}).index"
+                        //                         , sortX, sortY, sortX)
+                        // }
+                    } else {
+                        if (hue !== '' && sortHue !== '') {
+                            sortCode = com_util.formatString("{0}[{1}[{2}]=={3}].groupby({4})[{5}].mean().sort_values({6}).index"
+                                                , state.data, state.data, state.hue, com_util.convertToStr(sortHue, sortHueText), sortX, sortY, sortTypeStr);
+                        } else {
+                            sortCode = com_util.formatString("{0}.groupby({1})[{2}].mean().sort_values({3}).index", state.data, sortX, sortY, sortTypeStr);
+                        }
+                    }
+                } else if (chartType === 'countplot') {
+                    let countVar = sortX === ''? sortY: sortX;
+                    if (setXY === true) {
+                        // TODO: sort on setXY
+                        ;
+                    } else {
+                        if (hue !== '' && sortHue !== '') {
+                            sortCode = com_util.formatString("{0}[{1}[{2}]=={3}][{4}].value_counts({5}).index"
+                                                , state.data, state.data, state.hue, com_util.convertToStr(sortHue, sortHueText), countVar, sortTypeStr);
+                        } else {
+                            sortCode = com_util.formatString("{0}[{1}].value_counts({2}).index", state.data, countVar, sortTypeStr);
+                        }
+                    }
+                }
+
+                if (sortCode != '') {
+                    etcOptionCode.push('order=' + sortCode);
+                }
+            }
+
+            // add user option
+            if (userOption != '') {
+                etcOptionCode.push(userOption);
+            } 
 
             let generatedCode = com_generator.vp_codeGenerator(this, config, state
                 , etcOptionCode.length > 0? ', ' + etcOptionCode.join(', '): '');
@@ -851,7 +978,11 @@ define([
 
                 if (showValues && showValues === true) {
                     code.appendLine('ax = ' + generatedCode);
-                    code.appendLine("vp_seaborn_show_values(ax)");
+                    code.append("vp_seaborn_show_values(ax");
+                    if (showValuesPrecision !== '') {
+                        code.appendFormat(", precision={0}", showValuesPrecision);
+                    }
+                    code.appendLine(")");
                 } else {
                     code.appendLine(generatedCode);  
                 }
